function [g,w,c] = cg_gwc_aMRF(g,w,c,vx)
% Application of HMRF to segmented data using fast convolution filter
%
% FORMAT [g,w,c] = cg_gwc_aMRF(g,w,c,vx)
% g         - gray matter image (uint8 format)
% w         - white matter image (uint8 format)
% c         - CSF (uint8 format)
% vx        - voxel size to correct for anisotropy 
%
% Application of a Hidden Markov Random Field (HMRF) model introduces spatial
% constraints based on neighbouring voxels of a 3x3x3 cube. The center voxel has 26
% neighbours and we can calculate MRF energy by counting the number of neighbours.
% The idea is to remove isolated voxels of one tissue class which are unlikely to
% be member of this tissue type. This procedure also closes holes in a cluster
% of connected voxels of one tissue type. In the resulting segmentation the
% noise level will be minimized.
% The procedure is repeated until no changes below a threshold can be found. This
% iterative approach will filter noisy images by a larger amount (due to more iterations)
% than images without noise.
%_______________________________________________________________________
% @(#)cg_gwc_aMRF.m	1.01 Christian Gaser 2008/02/20

MRFbeta = 0.1;
diff_th = 0.004;

fprintf('Applying HMRF with iterative weighting\n');

sz = size(g);

% use only bounding box where sum of tissue classes is > 0.2 to save memory
th=255*0.2;
mask_gwc = double(g); % separate 'plus' steps to save memory
mask_gwc = mask_gwc + double(w);
mask_gwc = mask_gwc + double(c);
mask_gwc = mask_gwc > th;
sum_mask = sum(mask_gwc(:));

[indx, indy, indz] = ind2sub(size(mask_gwc),find(mask_gwc));
indx = max((min(indx) - 1),1):min((max(indx) + 1),sz(1));
indy = max((min(indy) - 1),1):min((max(indy) + 1),sz(2));
indz = max((min(indz) - 1),1):min((max(indz) + 1),sz(3));

% compute MRF energy of label and calculate joint probabilities 
% P(y,x) = P(y|x)*P(x)
% P(y|x) are already computed probablities of the tissue classes
% and P(x) = exp(-U(x)) is the HMRF prior probability of the classes

label = repmat(uint8(0),sz);
label2 = repmat(uint8(0),sz);

label(find((c >= g) & (c >= w))) = 1;
label(find((g >= c) & (g >= w))) = 2;
label(find((w >= c) & (w >= g))) = 3;

change = 1e10;
it = 0;

% repeat until change is below threshold or max. number of iterations is exceeded
while ((change > diff_th) & (it < 15))
	it = it + 1;

	% save old values
	g2 = g;	w2 = w;	c2 = c;

	% GM
	label2(:) = 0;
	label2(find((g >= c) & (g >= w))) = 1;
	PUx = get_PUx26(label2(indx,indy,indz),vx, MRFbeta);  
	g(indx,indy,indz) = uint8(round(double(g(indx,indy,indz)).*double(PUx)/255));

	% WM
	label2(:) = 0;
	label2(find((w >= c) & (w >= g))) = 1;
	PUx = get_PUx26(label2(indx,indy,indz),vx, MRFbeta);
	w(indx,indy,indz) = uint8(round(double(w(indx,indy,indz)).*double(PUx)/255));

	% CSF
	label2(:) = 0;
	label2(find((c >= g) & (c >= w))) = 1;
	PUx = get_PUx26(label2(indx,indy,indz),vx, MRFbeta);
	c(indx,indy,indz) = uint8(round(double(c(indx,indy,indz)).*double(PUx)/255));
	sum_gwc = double(g(indx,indy,indz))  + double(w(indx,indy,indz)) + double(c(indx,indy,indz));

	% Because we don't have the right normalization factor of the Gibbs distribution
	% we normalize by forcing that the sum of all joint probabilities is 1
	g(indx,indy,indz) = uint8(round(255*double(g(indx,indy,indz)).*mask_gwc(indx,indy,indz)./(sum_gwc + eps)));
	w(indx,indy,indz) = uint8(round(255*double(w(indx,indy,indz)).*mask_gwc(indx,indy,indz)./(sum_gwc + eps)));
	c(indx,indy,indz) = uint8(round(255*double(c(indx,indy,indz)).*mask_gwc(indx,indy,indz)./(sum_gwc + eps)));

	label2(:) = 0;
	label2(find((c >= g) & (c >= w))) = 1;
	label2(find((g >= c) & (g >= w))) = 2;
	label2(find((w >= c) & (w >= g))) = 3;

	% calculate change to previous iteration
	change = abs(double(label2)-double(label));
	% and relate it to number of voxels in mask
	change = sum(change(:))/sum_mask;
	
	% save old values
	label = label2;
	
end

fprintf('Iterative HMRF weighting with %g iterations\n',it);

% rescue values from previous iteration
g = g2; w = w2; c = c2;

return
%=======================================================================

%=======================================================================
function PUx = get_PUx26(label, vx, MRFbeta)
% Calculation of MRF probability of segmented data
%
% FORMAT PUx = get_PUx26(label, label_index, vx ,MRFbeta)
% label         - labeled segmentation ("1" for current tissue class, "0" for others) 
% vx            - voxel size to correct for anisotropy
% MRFbeta       - beta weighting of MRF energy
%
% Function to calculate prior MRF probability based on MRF energy U(x) according
% to the Ising model by using a convolution filter. The idea consists in using a 
% box of 3x3x3 voxels as filter kernel. Because we are using an image where the 
% voxels of the current tissue class are 1 and otherwise 0 this filter calculates 
% the sum of all voxels in the box.
% Distances between the voxel are taken into account by weighting the kernel
% with the invers of the voxel size.
%

% for anisotropic data we use voxel size to weight kernel according to the distances
% between neighboring voxels
dist = 1./([2 1 2].^(1/4));
kx = ones(1,3).*dist/vx(1);
ky = ones(1,3).*dist/vx(2);
kz = ones(1,3).*dist/vx(3);

% convolving data with kernel [1 1 1] weighted by dist which equals the sum of each label_index
% using 26 neighbours resulting in a maximum value of 9 for all 26 neighbours plus
% the center voxel
spm_conv_vol(label,label,kx,ky,kz,-[1 1 1]);

% maximum should be 9 to be compatible with beta weighting
PUx = 9*double(label)/max(double(label(:)));
clear label
PUx = exp(MRFbeta*PUx);

% normalize prior probability to 255 (not essential because we
% scale the sum of the joint probability to 1 at the end)
PUx = PUx/max(PUx(:));
PUx = uint8(round(255*PUx));

return
%=======================================================================
