#! /bin/sh

########################################################
# global parameters
########################################################
version='cg_vbm_batch.sh, v1.18 2008/08/20 Christian Gaser'

spm5=~/spm/spm5	# this parameter has to be set to your spm5 directory
matlab=matlab	# you can use other matlab versions by changing the matlab parameter
writeonly=0

########################################################
# run main
########################################################

main ()
{
  parse_args ${1+"$@"}
  check_matlab
  run_vbm

  exit 0
}


########################################################
# check arguments and files
########################################################

parse_args ()
{
  local optname optarg

  if [ $# -lt 1 ]; then
    help
    exit 1
  fi

  while [ $# -gt 0 ]
  do
	optname="`echo $1 | sed 's,=.*,,'`"
	optarg="`echo $2 | sed 's,^[^=]*=,,'`"
	case "$1" in
		--m* | -m*)
			exit_if_empty "$optname" "$optarg"
			matlab=$optarg
			shift
			;;
		--s* | -s*)
			exit_if_empty "$optname" "$optarg"
			spm5=$optarg
			shift
			;;
		--w* | -w*)
			writeonly=1
			;;
		-h | --help | -v | --version | -V)
			help
			exit 1
			;;
		-*)
			echo "`basename $0`: ERROR: Unrecognized option \"$1\"" >&2
			;;
		*)
			file="$1"
			;;
	esac
    shift
  done

}

########################################################
# check arguments
########################################################

exit_if_empty ()
{
  local desc val

  desc="$1"
  shift
  val="$*"

  if [ -z "$val" ]
  then
	echo 'ERROR: No argument given with \"$desc\" command line argument!' >&2
	exit 1
  fi
}

########################################################
# run vbm tool
########################################################

run_vbm ()
{
	cwd=`dirname $0`
	pwd=$PWD
	
	# we have to go into toolbox folder to find matlab files
	cd $cwd

	if [ $# -eq 2 ]; then
		if [ ! -d $spm5 ]; then
			spm5=${pwd}/$spm5
	    fi
		if [ ! -d $spm5 ]; then
			echo Directory $spm5 does not exist.
			exit 0
		fi
	fi

	export MATLABPATH=$MATLABPATH:${spm5}/toolbox/vbm5:$spm5
	
	time=`date "+%Y%b%d_%H%M"`
	vbmlog=${pwd}/vbm_${time}.log
	echo Check $vbmlog for logging information
	echo
	
	# correct filename
	for i in $file; do
		if [ ! -f $i ]; then
			file=${pwd}/$file
		fi
		break
	done

	X="cg_vbm_batch('${file}',${writeonly})"
	echo Calculate $file
	echo > $vbmlog
	echo ---------------------------------- >> $vbmlog
	date >> $vbmlog
	echo ---------------------------------- >> $vbmlog
	echo >> $vbmlog
	echo $0 $file >> $vbmlog
	echo >> $vbmlog
	nohup ${matlab} -nodisplay -nojvm -nosplash -r $X >> $vbmlog 2>&1 &

	exit 0
}

########################################################
# check if matlab exist
########################################################

check_matlab ()
{
  found=`which ${matlab} 2>/dev/null`
  if [ ! -n "$found" ];then
    echo $matlab not found.
    exit 1
  fi
}

########################################################
# help
########################################################

help ()
{
cat <<__EOM__

USAGE:
   cg_vbm_batch.sh filename|filepattern [-s spm5-path] [-m matlabcommand] [-w]
   
   -m   matlab command
   -s   spm5 directory
   -w
   Only one filename or pattern is allowed. This can be either a single file or a pattern
   with wildcards to process multiple files. For the latter case you have to (single) quote
   the pattern. Optionally you can set the spm5 directory with the "-s" option and the
   matlab command with the "-m" option and force to write already estimated segmentations with
   the "-w" option.

PURPOSE:
   Command line call of VBM5 segmentation

EXAMPLE
   cg_vbm_batch.sh spm/spm5/canonical/single_subj_T1.nii -s ~/spm/spm5
   This command will process only the single file single_subj_T1.nii. The spm5 directory
   is set to ~/spm/spm5.
   
   cg_vbm_batch.sh 'spm/spm5/canonical/*152*.nii'
   Using the quotes and wildcards all files containing the term "152" will
   be processed. In this case these are the files avg152PD.nii, avg152T1.nii,
   and avg152T2.nii.

   cg_vbm_batch.sh 'spm/spm5/canonical/*152*.nii' -m /usr/local/bin/matlab7
   Using the quotes and wildcards all files containing the term "152" will
   be processed. In this case these are the files avg152PD.nii, avg152T1.nii,
   and avg152T2.nii. As matlab command /usr/local/bin/matlab7 will be used.

INPUT:
   analyze/nifti files

OUTPUT:
   segmented images according to settings in cg_vbm_defaults.m
   vbm_log_$time.txt for log information

USED FUNCTIONS:
   cg_vbm_batch.m
   VBM5.1 toolbox
   SPM5

SETTINGS
   spm5 path: $spm5
   matlab command: $matlab
   
This script was written by Christian Gaser (christian.gaser@uni-jena.de).
This is version ${version}.

__EOM__
}

########################################################
# call main program
########################################################

main ${1+"$@"}
