function cg_vbm_write(p,job,ip)
% Write out VBM data
% FORMAT cg_vbm_write(p,opts)
% p    - results from spm_prep2sn
% job - writing options.  A struct containing these fields:
%        BIAS    - flags for bias corrected image
%        GM      - flags for which images should be written
%        WM      - similar to GM
%        CSF     - similar to GM
%        vox     - voxel size
%        bb      - bounding box
%        cleanup - clean up procedure
%        mrf     - Markov Random Fields approach
%        usepriors   - use tissue priors for writing data
%        writeaffine - write data using affine parameters only
% ip  - inverse parameters from spm_prep2sn
%____________________________________________________________________________
% Copyright (C) 2005 Wellcome Department of Imaging Neuroscience
%
% based on John Ashburners version of
% $Id: spm_preproc_write.m 716 2007-01-16 21:13:50Z karl $
%
% @(#)cg_vbm_write.m 1.11 2007/10/24 Christian Gaser

if nargin<2
	error('No options given.');
end;

if numel(p)>0,
    b0  = spm_load_priors(p(1).VG);
end;

for i=1:numel(p),
	if nargin == 3
	    vbm_apply(p(i),job,b0,ip(i));
	else
	    vbm_apply(p(i),job,b0);	
	end
end;
return;
%=======================================================================

%=======================================================================
function vbm_apply(p,job,b0,ip)

linfun = inline('fprintf(''%-70s%s'', x,repmat(sprintf(''\b''),1,70))');

opts = job.output;

% use old syntax for compatibility purposes
if isfield(opts.BIAS,'descalp')
	opts.BIAS = [opts.BIAS.native opts.BIAS.warped opts.BIAS.descalp];
end

% if descalp is not defined set to 0
if length(opts.BIAS) == 2
	opts.BIAS(3) = 0;
end

% the field 'subj' indicates longitudinal processing
if isfield(job,'subj')
	long = 1;
else
	long = 0;
end

% prevent that descalp option is used but neither native nor warped space is selected
% for writing bias corrected image
if opts.BIAS(3) & ~opts.BIAS(1) & ~opts.BIAS(2)
	opts.BIAS(3) = 0;
end

if opts.extopts.usepriors
	str = 'c';
else
	str = 'p';
end

% reorder options for compatibility issues
sopts = [[opts.GM.modulated opts.GM.warped opts.GM.native] ;...
		[opts.WM.modulated opts.WM.warped opts.WM.native] ;...
		[opts.CSF.modulated opts.CSF.warped opts.CSF.native]];

bb = opts.extopts.bb;

[pth,nam,ext]=fileparts(p.VF.fname);

T    = p.flags.Twarp;

bsol = p.flags.Tbias;
d2   = [size(T) 1];
d    = p.VF.dim(1:3);

[x1,x2,o] = ndgrid(1:d(1),1:d(2),1);
x3  = 1:d(3);
d3  = [size(bsol) 1];
B1  = spm_dctmtx(d(1),d2(1));
B2  = spm_dctmtx(d(2),d2(2));
B3  = spm_dctmtx(d(3),d2(3));
bB3 = spm_dctmtx(d(3),d3(3),x3);
bB2 = spm_dctmtx(d(2),d3(2),x2(1,:)');
bB1 = spm_dctmtx(d(1),d3(1),x1(:,1));

mg  = p.flags.mg;
mn  = p.flags.mn;
vr  = p.flags.vr;
K   = length(p.flags.mg);
Kb  = length(p.flags.ngaus);

% don't use background values for mask
Kb0  = Kb - 1;
K0   = sum(p.flags.ngaus(1:Kb0));

% prepare segmentations
for k1=1:size(sopts,1),
    dat{k1} = uint8(0);
    dat{k1}(d(1),d(2),d(3)) = 0;
    
    if sopts(k1,3),
        Vt        = struct('fname',   fullfile(pth,[str, num2str(k1), nam, ext]),...
                           'dim',     p.VF.dim,...
                           'dt',      [spm_type('uint8') spm_platform('bigend')],...
                           'pinfo',   [1/255 0 0]',...
                           'mat',     p.VF.mat,...
                           'n',       [1 1],...
                           'descrip', ['Tissue class ' num2str(k1)]);
        if opts.extopts.mrf                           
	        if (opts.extopts.mrf==1)
    	        Vt.fname = fullfile(pth,[str, num2str(k1), nam, '_aMRF', ext]);
        	    Vt.descrip = sprintf('aMRF class %d',k1);
        	else
            	Vt.fname = fullfile(pth,[str, num2str(k1), nam, '_HMRF', ext]);
            	Vt.descrip = sprintf('HMRF %g class %d',opts.extopts.mrf,k1);
        	end
        end
        Vt        = spm_create_vol(Vt);
        VO(k1)    = Vt;
    end;
end;

% prepare bias corrected image
if opts.BIAS(1),
    VB = struct('fname',   fullfile(pth,['m', nam, ext]),...
                'dim',     p.VF.dim(1:3),...
                'dt',      [spm_type('int16') spm_platform('bigend')],...
                'pinfo',   [1 0 0]',...
                'mat',     p.VF.mat,...
                'n',       [1 1],...
                'descrip', 'Bias Corrected');
	if opts.extopts.writeaffine, nam = [nam '_affine']; end
	if opts.BIAS(3), nam = [nam '_descalped']; end
   	VB.fname = fullfile(pth,['m', nam, ext]);
    VB = spm_create_vol(VB);
end;

lkp = []; for k=1:Kb, lkp = [lkp ones(1,p.flags.ngaus(k))*k]; end;

disp(['Working on ' nam])
linfun('Computing probabilities...');
spm_progress_bar('init',length(x3),['Working on ' nam],'Planes completed');
M = p.VG(1).mat\p.flags.Affine*p.VF.mat;

for z=1:length(x3),

    if (any(sopts(:)) | opts.BIAS(3))
	    % Bias corrected image
    	f          = spm_sample_vol(p.VF,x1,x2,o*x3(z),0);
    	% check that bias correction was really used
    	if ~isempty(bsol)
        	cr         = exp(transf(bB1,bB2,bB3(z,:),bsol)).*f;
    	else
        	cr = f;
    	end;
    	
        msk        = find(f<p.flags.thresh);
        [t1,t2,t3] = defs(T,z,B1,B2,B3,x1,x2,x3,M);
        q          = zeros([d(1:2) Kb]);
        
		if ~opts.extopts.usepriors
    	    brainmask  = zeros([d(1:2) 1]);
        	for k1=1:Kb0,
            	brainmask = brainmask + spm_sample_priors(b0{k1},t1,t2,t3,k1==Kb);
        	end;
        	brainmask = brainmask > 0.1;
		end
       	if opts.extopts.usepriors
	    	bt         = zeros([d(1:2) Kb]);
    	    for k1=1:Kb
        	    bt(:,:,k1) = spm_sample_priors(b0{k1},t1,t2,t3,k1==Kb);
        	end

	    	b = zeros([d(1:2) K]);
	        for k=1:K, b(:,:,k) = bt(:,:,lkp(k))*mg(k); end;
	    	s = sum(b,3)+eps;
	    end
        for k=1:K,
            p1            = exp((cr-mn(k)).^2/(-2*vr(k)))/sqrt(2*pi*vr(k)+eps);
	       	if opts.extopts.usepriors
	       		p1 = p1.*b(:,:,k)./s;
			else
%				p1 = p1.*brainmask;
	       		p1 = p1.*brainmask*mg(k);
			end
    	    q(:,:,lkp(k)) = q(:,:,lkp(k)) + p1;
        end;
        sq = sum(q,3)+eps;
        
        warning off
        for k1=1:size(sopts,1),
            tmp            = q(:,:,k1);
            tmp(msk)       = 0;
            tmp            = tmp./sq;
            dat{k1}(:,:,z) = uint8(round(255 * tmp));

            % write only if mrf option is not selected
            if sopts(k1,3) & ~opts.extopts.mrf,
                spm_write_plane(VO(k1),tmp,z);
            end;
        end;
        warning on
    end;
    spm_progress_bar('set',z);
end;
spm_progress_bar('clear');

clear b0 q b bt cr f p1 s sq t1 t2 t3 brainmask

% use only bounding box where sum of tissue classes is > 0.05 to save memory
if (opts.extopts.cleanup | opts.extopts.mrf) & (any(sopts(:) | opts.BIAS(3)))
	sz = size(dat{1});

	th=255*0.05;
	mask_gwc = double(dat{1}); % separate 'plus' steps to save memory
	mask_gwc = mask_gwc + double(dat{2});
	mask_gwc = mask_gwc + double(dat{3});
	mask_gwc = mask_gwc > th;

	[indx, indy, indz] = ind2sub(size(mask_gwc),find(mask_gwc));
	clear mask_gwc;
	indx = max((min(indx) - 1),1):min((max(indx) + 1),sz(1));
	indy = max((min(indy) - 1),1):min((max(indy) + 1),sz(2));
	indz = max((min(indz) - 1),1):min((max(indz) + 1),sz(3));
end

% clean up
if opts.extopts.cleanup & (any(sopts(:)) | opts.BIAS(3))
    linfun('Clean up...');
    [dat{1}(indx,indy,indz), dat{2}(indx,indy,indz), dat{3}(indx,indy,indz)] = clean_gwc(dat{1}(indx,indy,indz), ...
    	dat{2}(indx,indy,indz), dat{3}(indx,indy,indz), opts.extopts.cleanup);
end;

% Markov Random Field approach
if opts.extopts.mrf & (any(sopts(:)) | opts.BIAS(3))
    if opts.extopts.mrf==1
	    linfun(sprintf('Computing HMRF with iterative beta ...'));
	else
	    linfun(sprintf('Computing HMRF with beta %g ...', opts.extopts.mrf));
	end
    vx   = sqrt(sum(p.VF.mat(1:3,1:3).^2));
    if opts.extopts.mrf==1
	    [dat{1}(indx,indy,indz),dat{2}(indx,indy,indz),dat{3}(indx,indy,indz)] = cg_gwc_aMRF(dat{1}(indx,indy,indz),...
    		 dat{2}(indx,indy,indz), dat{3}(indx,indy,indz), vx);
	else
	    [dat{1}(indx,indy,indz),dat{2}(indx,indy,indz),dat{3}(indx,indy,indz)] = cg_gwc_HMRF(dat{1}(indx,indy,indz),...
    		 dat{2}(indx,indy,indz), dat{3}(indx,indy,indz), opts.extopts.mrf, vx);
	end
    for k1=1:size(sopts,1),
        % write only if mrf option was selected
        if sopts(k1,3)
            tmp = double(dat{k1})/255;
            for z=1:length(x3),
                spm_write_plane(VO(k1),tmp(:,:,z),z);
            end
        end;
    end;
end;

% reduce warps to affine registration
if opts.extopts.writeaffine
	p.Tr    = [];
	% because the affine information is lost we use inverse affine matrix from the *seg_inv_sn.mat file
	p.Affine = inv(ip.Affine);
end

% write bias corrected images
if opts.BIAS(2) | opts.BIAS(1)
    linfun('Writing warped bias corrected image...');
    dim     = p.VF.dim;
    cr_vol = zeros(dim);
    
	for z=1:length(x3),

    	% Bias corrected image
    	f          = spm_sample_vol(p.VF,x1,x2,o*x3(z),0);
    	% check that bias correction was really used
    	if ~isempty(bsol)
        	cr         = exp(transf(bB1,bB2,bB3(z,:),bsol)).*f;
    	else
        	cr = f;
    	end;
    	cr_vol(:,:,z) = cr;
    end

    if opts.BIAS(3)
    	% calculate sum in single steps to save menory
    	mask_descalp = double(dat{1});
    	mask_descalp = mask_descalp + double(dat{2});
    	mask_descalp = mask_descalp + double(dat{3});
		mask_descalp = mask_descalp > 0.75*255;
		mask_descalp = cg_morph_vol(mask_descalp,'close',2,0.75);
        cr_vol = cr_vol.*double(mask_descalp);
        clear mask_descalp
    end;

    if opts.BIAS(1)
        VB = spm_write_vol(VB,cr_vol);
    end;

    if opts.BIAS(2)
		% scale image to maximum to write it as int16
		int_scale = 32767/max(max(max(cr_vol)));
		cr_vol = int16(round(int_scale*cr_vol));
    	so      = struct('wrap',[0 0 0],'interp',2,'vox',opts.extopts.vox,...
                         'bb',bb,'preserve',0);
        if opts.extopts.writeaffine
        	if isempty(strfind(nam,'_affine'))
        		nam = [nam '_affine'];
        	end
        end
    	VNB = struct('fname',   fullfile(pth,['m', nam, ext]),...
                'dim',     p.VF.dim(1:3),...
                'dt',      [spm_type('int16') spm_platform('bigend')],...
                'pinfo',   [1/int_scale 0 0]',...
                'mat',     p.VF.mat,...
                'dat',	cr_vol,...
                'descrip', 'Normalised Bias Corrected');
    	if opts.BIAS(3)
    		VNB.fname = fullfile(pth,['m', nam, '_descalped', ext]);
    	end

    	spm_write_sn(VNB,p,so);
    end
end

% write normalized segmentations
for k1=1:size(sopts,1),
    if any(sopts(k1,1:2)),
        linfun('Writing images...');
        so      = struct('wrap',[0 0 0],'interp',2,'vox',opts.extopts.vox,...
                         'bb',bb,'preserve',0);
        ovx     = abs(det(p.VG(1).mat(1:3,1:3)))^(1/3);
        fwhm    = max(ovx./sqrt(sum(p.VF.mat(1:3,1:3).^2))-1,0.1);
%        dat{k1} = decimate(dat{k1},fwhm);
        dim     = [size(dat{k1}) 1];
        VT      = struct('dim',dim(1:3),...
                     'dt', [spm_type('uint8') spm_platform('bigend')],...
                     'pinfo',[1/255 0 0]',...
                     'mat',p.VF.mat,...
                     'dat',dat{k1});
        oname = nam;
		if opts.extopts.writeaffine
			if isempty(strfind(oname,'_affine'))
				oname = [oname '_affine'];
			end
		end
        if long, oname = [oname '_long']; end
        if opts.extopts.mrf
	        if (opts.extopts.mrf==1)
        		oname = [oname '_aMRF'];
			    VT.descrip = sprintf('aMRF class %d',k1);
        	else
        		oname = [oname '_HMRF'];
			    VT.descrip = sprintf('HMRF %g class %d',opts.extopts.mrf,k1);
        	end
        end
	    VT.fname = fullfile(pth,[str, num2str(k1), oname, ext]);
        if sopts(k1,2),
            spm_write_sn(VT,p,so);
        end;

		% modulation
        so.preserve = 1;
        if sopts(k1,1),
           VN       = spm_write_sn(VT,p,so);
           VN.fname = fullfile(pth,['mw', str, num2str(k1), oname, ext]);
	       VN.descrip = sprintf('class %d modulated',k1);
	       VN.dt = [spm_type('float32') spm_platform('bigend')];
	       VN.pinfo = [1 0 0]';
	       % removing of affine scaling in modulation
	       if sopts(k1,1) == 2
	           VN.fname = fullfile(pth,['m0w', str, num2str(k1), oname, ext]);
		       VN.descrip = sprintf('class %d modulated (warp only)',k1);
		       M0 = ip.VG(1).mat*inv(ip.Affine)\ip.VF.mat;
	    	   detAff = det(M0(1:3,1:3));
	    	   VN.dat = VN.dat*detAff;
		   end		    	   
           spm_write_vol(VN,VN.dat);
        end;
    end;
end;

% save tissue class volumes in ml in log-file
volfactor = abs(det(p.VF.mat(1:3,1:3)))/1000;
vol_txt = fullfile(pth,[str, nam, '_seg.txt']);
fid = fopen(vol_txt, 'w');
for i=1:size(sopts,1)
    vol = volfactor*sum(dat{i}(:))/255;	
    fprintf(fid,'%5.3f\t',vol);
end;
fclose(fid);

% display and print result if possible
try
  if any(sopts(:)) & opts.extopts.print
	str = [];
	str = [str struct('name', 'Gaussians:','value',sprintf('%d %d %d %d',p.flags.ngaus))];
	str = [str struct('name', 'Warp cutoff:','value',sprintf('%d',p.flags.warpco))];
	str = [str struct('name', 'Warp regularisation:','value',sprintf('%g',p.flags.warpreg))];
	str = [str struct('name', 'Bias regularisation:','value',sprintf('%g',p.flags.biasreg))];
	str = [str struct('name', 'Bias FWHM:','value',sprintf('%d',p.flags.biasfwhm))];
	str = [str struct('name', 'Clean up:','value',sprintf('%d',opts.extopts.cleanup))];
	if (opts.extopts.mrf==1)
		str = [str struct('name', 'Iterative HMRF:','value',sprintf('%g',0.1))];
	else
		str = [str struct('name', 'HMRF:','value',sprintf('%g',opts.extopts.mrf))];
	end
	str = [str struct('name', 'Write affine only:','value',sprintf('%d',opts.extopts.writeaffine))];
	str = [str struct('name', 'Use tissue priors:','value',sprintf('%d',opts.extopts.usepriors))];
	if long
		str = [str struct('name', 'Longitudinal processing','value','')];
	end

	fg = spm_figure('FindWin','Graphics');
	spm_figure('Clear','Graphics');
	ax=axes('Position',[0.01 0.75 0.98 0.23],'Visible','off','Parent',fg);
	text(0,0.95,  ['Segmentation: ' spm_str_manip(p.VF.fname,'k50d')],'FontSize',12,'FontWeight','Bold',...
		'Interpreter','none','Parent',ax);
	for i=1:size(str,2)
		text(0.05,0.85-(0.075*i), str(i).name ,'FontSize',12, 'Interpreter','none','Parent',ax);
		text(0.35,0.85-(0.075*i), str(i).value ,'FontSize',12, 'Interpreter','none','Parent',ax);
	end
	pos = [0.01 0.3 0.48 0.6; 0.51 0.3 0.48 0.6; ...
			0.01 -0.1 0.48 0.6; 0.51 -0.1 0.48 0.6];
	spm_orthviews('Reset');
	% first try use the bias corrected image
	try
    	Vtmp = spm_write_sn(VNB,p);
	catch
	    Vtmp = spm_write_sn(p.VF,p);
	end
	spm_orthviews('Image',Vtmp,pos(1,:));
	for k1=1:size(sopts,1),
        dim     = [size(dat{k1}) 1];
        VT      = struct('dim',dim(1:3),...
                     'dt', [spm_type('uint8') spm_platform('bigend')],...
                     'pinfo',[1/255 0 0]',...
                     'mat',p.VF.mat,...
                     'fname', 'tmp.img',...
                     'dat',dat{k1});
        Vtmp = spm_write_sn(VT,p);
		spm_orthviews('Image',Vtmp,pos(1+k1,:));
	end
	spm_print;
  end
end

return;
%=======================================================================

%=======================================================================
function [x1,y1,z1] = defs(sol,z,B1,B2,B3,x0,y0,z0,M)
x1a = x0    + transf(B1,B2,B3(z,:),sol(:,:,:,1));
y1a = y0    + transf(B1,B2,B3(z,:),sol(:,:,:,2));
z1a = z0(z) + transf(B1,B2,B3(z,:),sol(:,:,:,3));
x1  = M(1,1)*x1a + M(1,2)*y1a + M(1,3)*z1a + M(1,4);
y1  = M(2,1)*x1a + M(2,2)*y1a + M(2,3)*z1a + M(2,4);
z1  = M(3,1)*x1a + M(3,2)*y1a + M(3,3)*z1a + M(3,4);
return;
%=======================================================================

%=======================================================================
function t = transf(B1,B2,B3,T)
if ~isempty(T)
    d2 = [size(T) 1];
    t1 = reshape(reshape(T, d2(1)*d2(2),d2(3))*B3', d2(1), d2(2));
    t  = B1*t1*B2';
else
    t = zeros(size(B1,1),size(B2,1),size(B3,1));
end;
return;
%=======================================================================

%=======================================================================
function dat = decimate(dat,fwhm)
% Convolve the volume in memory (fwhm in voxels).
lim = ceil(2*fwhm);
x  = -lim(1):lim(1); x = spm_smoothkern(fwhm(1),x); x  = x/sum(x);
y  = -lim(2):lim(2); y = spm_smoothkern(fwhm(2),y); y  = y/sum(y);
z  = -lim(3):lim(3); z = spm_smoothkern(fwhm(3),z); z  = z/sum(z);
i  = (length(x) - 1)/2;
j  = (length(y) - 1)/2;
k  = (length(z) - 1)/2;
spm_conv_vol(dat,dat,x,y,z,-[i j k]);
return;
%=======================================================================

%=======================================================================
function y = mask_largest_cluster(y, th)

if nargin < 2
	th = 0.5;
end

sz = size(y);

th = th*max(double(y(:)));

mask = y > th;
Q = find(mask);

Qth = find(y <= th & y>0);
yth = y(Qth);

% save memory by using bounding box where y > th
[indx, indy, indz] = ind2sub(size(mask),Q);
indx = max((min(indx) - 1),1):min((max(indx) + 1),sz(1));
indy = max((min(indy) - 1),1):min((max(indy) + 1),sz(2));
indz = max((min(indz) - 1),1):min((max(indz) + 1),sz(3));

[A,num] = spm_bwlabel(double(mask(indx,indy,indz)),26);

clear mask

% interrupt if cluster was > 7.5% of whole image to save time
max_A = max(A(:));
sz_cluster = zeros(max_A,1);
for i=1:max_A
	QA = find(A == i);
	ind = i;
	if length(QA)/prod(size(A)) > 0.075
		break
	end
	sz_cluster(i) = length(QA);
end

if length(QA)/prod(size(A)) <= 0.075
	[mx, ind] = max(sz_cluster);
	QA = find(A == ind);
end

QA0 = find(A ~= ind);
A = y(indx,indy,indz);
A(QA0) = 0;

y(indx,indy,indz) = A;
y(Qth) = yth;

return
%=======================================================================
function [g,w,c] = clean_gwc(g,w,c, level)

if nargin<4, level = 1; end;

th0 = 0.6;

mx_w = max(double(w(:)));

if level==2
	th1 = 0.2*mx_w/255;
else
	th1 = 0.15*mx_w/255;
end;

th0 = th0*mx_w/255;

% use only largest WM cluster at th0 as seed parameter
% mask out all voxels where wm is lower than gm or csf
wmask = w;
wmask(find((double(w) < double(c)) & (double(w) < double(g)))) = 0;
b = mask_largest_cluster(wmask, th0);

% Build a 3x3x3 seperable smoothing kernel
%-----------------------------------------------------------------------
kx=[0.75 1 0.75];
ky=[0.75 1 0.75];
kz=[0.75 1 0.75];
sm=sum(kron(kron(kz,ky),kx))^(1/3);
kx=kx/sm; ky=ky/sm; kz=kz/sm;

% Erosions and conditional dilations
%-----------------------------------------------------------------------

niter = 45;
spm_progress_bar('Init',niter,'Clean up','Iterations completed');

for j=1:niter,
        if j>2, th=th1; else th=th0; end; % Dilate after two its of erosion.
        for i=1:size(b,3),
                gp = double(g(:,:,i));
                wp = double(w(:,:,i));
                bp = double(b(:,:,i))/255;
                bp = (bp>th).*(wp+gp);
                b(:,:,i) = uint8(round(bp));
        end;
        spm_conv_vol(b,b,kx,ky,kz,-[1 1 1]);
        spm_progress_bar('Set',j);
end;

th = 0.05;

b2 = double(w);
b2 = b2 + double(g);
b2 = b2 + double(c);
b2 = cg_morph_vol(b2,'open',1,th1);
b2 = mask_largest_cluster(b2,0.5);
b2 = double(cg_morph_vol(b2,'close',2,0.5));

for i=1:size(b,3),
        gp       = double(g(:,:,i))/255;
        wp       = double(w(:,:,i))/255;
        cp       = double(c(:,:,i))/255;
        bp       = double(b(:,:,i))/255;
        bp       = ((bp>th).*(wp+gp))>th;
        gwc      = gp + wp + cp + eps;
        g(:,:,i) = uint8(round(255*gp.*bp./gwc));
        w(:,:,i) = uint8(round(255*wp.*bp./gwc));
        c(:,:,i) = uint8(round(255*cp.*b2(:,:,i)./gwc));
end;


spm_progress_bar('Clear');
return;
